@@ -4,7 +4,8 @@ use warnings;
 use Mouse;
 use AI::MXNet::Function::Parameters;
 use AI::MXNet::Logging;
-use overload "&{}" => sub { my $self = shift; sub { $self->call(@_) } };
+use overload "&{}" => sub { my $self = shift; sub { $self->call(@_) } },
+             fallback => 1;
 
 =head1 NAME
 
@@ -67,7 +68,7 @@ use Mouse;
 extends 'AI::MXNet::LRScheduler';
 
 has 'step'            => (is => 'ro', isa => 'Int', required => 1);
-has 'factor'          => (is => 'ro', isa => 'Int', default  => 1);
+has 'factor'          => (is => 'ro', isa => 'Num', default  => 1);
 has 'count'           => (is => 'rw', isa => 'Int', default  => 1);
 has 'stop_factor_lr'  => (is => 'ro', isa => 'Num', default  => 1e-8);
 
@@ -98,7 +99,7 @@ method call(Int $num_update)
         else
         {
             AI::MXNet::Logging->info(
-                "Update[%d]: Change learning rate to %0.5e",
+                "Update[%d]: Changed learning rate to %0.5e",
                 $num_update, $self->base_lr
             );
         }
@@ -129,8 +130,8 @@ package AI::MXNet::MultiFactorScheduler;
 use Mouse;
 extends 'AI::MXNet::LRScheduler';
 has 'step'            => (is => 'ro', isa => 'ArrayRef[Int]', required => 1);
-has 'factor'          => (is => 'ro', isa => 'Int', default  => 1);
-has 'cur_step_ind '   => (is => 'ro', isa => 'Int', default  => 0);
+has 'factor'          => (is => 'ro', isa => 'Num', default  => 1);
+has 'cur_step_ind'    => (is => 'rw', isa => 'Int', default  => 0);
 has 'count'           => (is => 'rw', isa => 'Int', default  => 0);
 
 sub BUILD
@@ -160,7 +161,7 @@ method call(Int $num_update)
             $self->cur_step_ind($self->cur_step_ind + 1);
             $self->base_lr($self->base_lr * $self->factor);
             AI::MXNet::Logging->info(
-                "Update[%d]: Change learning rate to %0.5e",
+                "Update[%d]: Changed learning rate to %0.5e",
                 $num_update, $self->base_lr
             );
         }
